<?php

namespace Drupal\m4032404\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteBuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the 403 to 404 module configuration form.
 *
 * @package Drupal\m4032404\Form
 */
class M4032404Form extends ConfigFormBase {

  /**
   * The route builder service.
   *
   * @var \Drupal\Core\Routing\RouteBuilderInterface
   */
  protected $routeBuilder;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'm4032404_config_form';
  }

  /**
   * Constructs a new M4032404Form object.
   *
   * @param \Drupal\Core\Routing\RouteBuilderInterface $route_builder
   *   The route builder service.
   */
  public function __construct(RouteBuilderInterface $route_builder) {
    $this->routeBuilder = $route_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('router.builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $settings = $this->config('m4032404.settings');

    $form['admin_only'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enforce on Admin Only'),
      '#description' => $this->t('Enforce the 403 to 404 behavior only on admin paths'),
      '#default_value' => (int) $settings->get('admin_only'),
    ];

    $form['pages'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Pages'),
      '#default_value' => implode("\r\n", $settings->get('pages') ?? []),
      '#description' => $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. An example path is %user-wildcard for every user page. %front is the front page.", [
        '%user-wildcard' => '/user/*',
        '%front' => '<front>',
      ]),
    ];

    $form['negate'] = [
      '#type' => 'radios',
      '#default_value' => (int) $settings->get('negate'),
      '#title_display' => 'invisible',
      '#options' => [
        $this->t('Redirect the above paths to 404'),
        $this->t('Do not redirect the above paths to 404'),
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save configuration'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('m4032404.settings')
      ->set('admin_only', $form_state->getValue('admin_only'))
      ->set('pages', explode("\r\n", $form_state->getValue('pages')))
      ->set('negate', (bool) $form_state->getValue('negate'))
      ->save();

    $this->routeBuilder->rebuild();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return ['m4032404.settings'];
  }

}
